%global sover  20
Summary:       A general purpose cryptographic library originally based on code from GnuPG
Name:          libgcrypt
Version:       1.10.2
Release:       5%{?dist}
License:       LGPLv2.1+
URL:           https://www.gnupg.org/software/libgcrypt/index.html
Source0:       https://www.gnupg.org/ftp/gcrypt/libgcrypt/libgcrypt-%{version}.tar.bz2

# Fix CVE-2024-2236
# Patches from: https://gitlab.com/redhat/centos-stream/rpms/libgcrypt/-/commit/c54e9e9ce8c8
Patch0001:     libgcrypt-1.10.0-marvin.patch
Patch0002:     libgcrypt-1.10.0-marvin2.patch
# https://gitlab.com/redhat-crypto/libgcrypt/libgcrypt-mirror/-/merge_requests/19/
Patch0003:     libgcrypt-1.10.0-marvin3.patch

Patch3000:     libgcrypt-1.10.0-disable-brainpool.patch

BuildRequires: gcc make autoconf automake libtool pkgconfig
BuildRequires: libgpg-error-devel >= 1.11
BuildRequires: gawk texinfo

%description
Libgcrypt is a general purpose cryptographic library originally based on code from GnuPG.
It provides functions for all cryptograhic building blocks: symmetric cipher algorithms
(AES, Arcfour, Blowfish, Camellia, CAST5, ChaCha20 DES, GOST28147, Salsa20, SEED, Serpent, Twofish)
and modes (ECB,CFB,CBC,OFB,CTR,CCM,GCM,OCB,POLY1305,AESWRAP), hash algorithms (MD2, MD4, MD5,
GOST R 34.11, RIPE-MD160, SHA-1, SHA2-224, SHA2-256, SHA2-384, SHA2-512, SHA3-224, SHA3-256, SHA3-384,
SHA3-512, SHAKE-128, SHAKE-256, TIGER-192, Whirlpool), MACs (HMAC for all hash algorithms,
CMAC for all cipher algorithms, GMAC-AES, GMAC-CAMELLIA, GMAC-TWOFISH, GMAC-SERPENT, GMAC-SEED,
Poly1305, Poly1305-AES, Poly1305-CAMELLIA, Poly1305-TWOFISH, Poly1305-SERPENT, Poly1305-SEED),
public key algorithms (RSA, Elgamal, DSA, ECDSA, EdDSA, ECDH), large integer functions,
random numbers and a lot of supporting functions.


%package devel
Summary:       Development files for libgcrypt
License:       LGPLv2+ and GPLv2+
Requires:      %{name} = %{version}-%{release}
Requires:      libgpg-error-devel pkgconfig

%description devel
Files to develop applications using libgcrypt.


%prep
%autosetup -p1


%build
%define _lto_cflags -flto=auto -ffat-lto-objects

autoreconf -f
%configure --disable-static \
           --enable-noexecstack \
           --enable-hmac-binary-check \
           --disable-brainpool \
           --enable-marvin-workaround \
           --with-fips-module-version="Libgcrypt version %{version}-%{release}"
sed -i -e '/^sys_lib_dlsearch_path_spec/s,/lib /usr/lib,/usr/lib /lib64 /usr/lib64 /lib,g' libtool
%make_build


# Add generation of HMAC checksums of the final stripped binaries 
%define libpath %{buildroot}%{_libdir}/%{name}.so.%{sover}.?.?	
%define __spec_install_post \
    %{?__debug_package:%{__debug_install_post}} \
    %{__arch_install_post} \
    %{__os_install_post} \
    pushd src \
    sed -i -e 's|FILE=.*|FILE=\\\$1|' gen-note-integrity.sh \
    READELF=readelf AWK=awk ECHO_N="-n" bash gen-note-integrity.sh %{libpath} > %{libpath}.hmac \
    objcopy --update-section .note.fdo.integrity=%{libpath}.hmac %{libpath} new \
    mv -f new %{libpath} \
    rm -f %{libpath}.hmac \
%{nil}

%install
%make_install
rm -f %{buildroot}%{_infodir}/dir
rm -f %{buildroot}%{_libdir}/*.la

sed -i -e 's,^libdir="/usr/lib.*"$,libdir="/usr/lib",g' %{buildroot}%{_bindir}/libgcrypt-config
sed -i -e 's,^my_host=".*"$,my_host="none",g' %{buildroot}%{_bindir}/libgcrypt-config

pushd %{buildroot}%{_libdir}
ln -sf %{name}.so.%{sover} %{buildroot}%{_libdir}/%{name}.so
popd

install -p -d -m 755 %{buildroot}%{_sysconfdir}/gcrypt


%check
make check
LIBGCRYPT_FORCE_FIPS_MODE=1 make check


%files
%license COPYING.LIB
%doc AUTHORS NEWS THANKS
%dir %{_sysconfdir}/gcrypt
%{_libdir}/libgcrypt.so.*

%files devel
%license COPYING COPYING.LIB
%doc README ChangeLog TODO
%{_bindir}/%{name}-config
%{_bindir}/dumpsexp
%{_bindir}/hmac256
%{_bindir}/mpicalc
%{_includedir}/*
%{_libdir}/*.so
%{_libdir}/pkgconfig/libgcrypt.pc
%{_datadir}/aclocal/*
%{_mandir}/man1/*
%{_infodir}/gcrypt.info*


%changelog
* Thu Nov 21 2024 Miaojun Dong <zoedong@tencent.com> - 1.10.2-5
- Fix CVE-2024-2236

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.10.2-4
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.10.2-3
- Rebuilt for loongarch release

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.10.2-2
- Rebuilt for OpenCloudOS Stream 23.09

* Thu Aug 03 2023 Miaojun Dong <zoedong@tencent.com> - 1.10.2-1
- Bump version to 1.10.2

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.10.1-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.10.1-2
- Rebuilt for OpenCloudOS Stream 23

* Mon Jul 11 2022 rockerzhu <rockerzhu@tencent.com> - 1.10.1-1
- Initial build
